'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _position = require('../position');

var _isWithin = require('../is-within');

var _isWithin2 = _interopRequireDefault(_isWithin);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getCorners = function getCorners(droppable) {
  var fragment = droppable.page.withMargin;

  return [{ x: fragment.left, y: fragment.top }, { x: fragment.right, y: fragment.top }, { x: fragment.left, y: fragment.bottom }, { x: fragment.right, y: fragment.bottom }];
};

exports.default = function (_ref) {
  var isMovingForward = _ref.isMovingForward,
      pageCenter = _ref.pageCenter,
      source = _ref.source,
      droppables = _ref.droppables;

  var axis = source.axis;

  var candidates = (0, _keys2.default)(droppables).map(function (id) {
    return droppables[id];
  }).filter(function (droppable) {
    return droppable !== source;
  }).filter(function (droppable) {
    return droppable.isEnabled;
  }).filter(function (droppable) {
    if (isMovingForward) {
      return source.page.withMargin[axis.crossAxisEnd] <= droppable.page.withMargin[axis.crossAxisStart];
    }

    return droppable.page.withMargin[axis.crossAxisEnd] <= source.page.withMargin[axis.crossAxisStart];
  }).filter(function (droppable) {
    var sourceFragment = source.page.withMargin;
    var destinationFragment = droppable.page.withMargin;

    var isBetweenSourceBounds = (0, _isWithin2.default)(sourceFragment[axis.start], sourceFragment[axis.end]);
    var isBetweenDestinationBounds = (0, _isWithin2.default)(destinationFragment[axis.start], destinationFragment[axis.end]);

    return isBetweenSourceBounds(destinationFragment[axis.start]) || isBetweenSourceBounds(destinationFragment[axis.end]) || isBetweenDestinationBounds(sourceFragment[axis.start]) || isBetweenDestinationBounds(sourceFragment[axis.end]);
  }).filter(function (droppable) {
    return droppable.page.withoutMargin[axis.crossAxisStart] >= droppable.container.bounds[axis.crossAxisStart] && droppable.page.withoutMargin[axis.crossAxisEnd] <= droppable.container.bounds[axis.crossAxisEnd];
  }).sort(function (a, b) {
    var first = a.page.withMargin[axis.crossAxisStart];
    var second = b.page.withMargin[axis.crossAxisStart];

    if (isMovingForward) {
      return first - second;
    }
    return second - first;
  }).filter(function (droppable, index, array) {
    return droppable.page.withMargin[axis.crossAxisStart] === array[0].page.withMargin[axis.crossAxisStart];
  });

  if (!candidates.length) {
    return null;
  }

  if (candidates.length === 1) {
    return candidates[0];
  }

  var contains = candidates.filter(function (droppable) {
    var isWithinDroppable = (0, _isWithin2.default)(droppable.page.withMargin[axis.start], droppable.page.withMargin[axis.end]);
    return isWithinDroppable(pageCenter[axis.line]);
  });

  if (contains.length === 1) {
    return contains[0];
  }

  if (contains.length > 1) {
    return contains.sort(function (a, b) {
      return a.page.withMargin[axis.start] - b.page.withMargin[axis.start];
    })[0];
  }

  return candidates.sort(function (a, b) {
    var first = (0, _position.closest)(pageCenter, getCorners(a));
    var second = (0, _position.closest)(pageCenter, getCorners(b));

    if (first !== second) {
      return first - second;
    }

    return a.page.withMargin[axis.start] - b.page.withMargin[axis.start];
  })[0];
};